"""
Entry-point module:

You can import any of the relevant objects by simply:
>>> from masked_norm import LazyAffineMaskedNorm
"""

from .functional import masked_norm
from .nn import MaskedNorm
from .nn import LazyMaskedNorm

from .functional import affine_masked_norm
from .nn import LazyAffineMaskedNorm
